package jaxx.runtime.swing;

import java.awt.Container;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import jaxx.runtime.JAXXObject;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Une extension de {@link CardLayout2} pour pouvoir automatiquement afficher un
 * contenu à partir de la propriété {@link #selected}.
 *
 * Ainsi, en changeant cette propriété via la méthode {@link #setSelected(String)},
 * le contenu sera changé automatiquement, ce qui permet une utilisation direct
 * dans jaxx sans à avoir à écrire d'écouteur.
 *
 * @author tony
 * @since 1.3
 * @see CardLayout2
 */
public class CardLayout2Ext extends CardLayout2 {

    /** log */
    static private Log log = LogFactory.getLog(CardLayout2.class);
    private static final long serialVersionUID = 1L;
    public static final String SELECTED_PROPERTY_NAME = "selected";
    /**
     * pour propager les changements dans le modèle  vers l'ui
     */
    protected PropertyChangeSupport pcs;
    /**
     * le contenu sélectionné
     */
    protected String selected;
    private String containerName;
    private JAXXObject ui;
    private Container container;

    public CardLayout2Ext(JAXXObject ui, String containerName) {
        pcs = new PropertyChangeSupport(this);
        this.ui = ui;
        this.containerName = containerName;
    }

    public String getSelected() {
        return selected;
    }

    public String getPreviousSelected() {
        int index = contexts.indexOf(selected);
        if (index < 1) {
            return null;
        }
        return contexts.get(index - 1) + "";
    }

    public String getNextSelected() {
        int index = contexts.indexOf(selected);
        if (index >= contexts.size()) {
            return null;
        }
        return contexts.get(index + 1) + "";
    }

    public void setSelected(String selected) {
        this.selected = selected;
        show(getContainer(), selected);
    }

    public Container getContainer() {
        if (container == null) {
            container = (Container) ui.getObjectById(containerName);
        }
        return container;
    }

    public void addPropertyChangeListener(PropertyChangeListener listener) {
        pcs.addPropertyChangeListener(listener);
    }

    public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        pcs.addPropertyChangeListener(propertyName, listener);
    }

    public void removePropertyChangeListener(PropertyChangeListener listener) {
        pcs.removePropertyChangeListener(listener);
    }

    public void removePropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        pcs.removePropertyChangeListener(propertyName, listener);
    }

    public void removePropertyChangeListeners() {
        for (PropertyChangeListener l : pcs.getPropertyChangeListeners()) {
            pcs.removePropertyChangeListener(l);
        }
    }
}
