package jaxx.runtime.validator.swing;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;
import java.awt.Component;
import jaxx.runtime.validator.BeanValidatorScope;
import jaxx.runtime.SwingValidatorUtil;
import static org.nuiton.i18n.I18n._;

/**
 * A simple render of a table of validator's messages, says a table that use
 * a {@link SwingValidatorMessageTableModel} model.
 *
 * @author chemit
 * @since 1.3
 * @see SwingValidatorMessageTableModel
 */
public class SwingValidatorMessageTableRenderer extends DefaultTableCellRenderer {

    private static final long serialVersionUID = 1L;

    @Override
    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
        JLabel rendererComponent = (JLabel) super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);

        ImageIcon icon = null;
        String text = null;
        String toolTipText = null;

        column = table.convertColumnIndexToModel(column);
        if (table.getRowSorter() != null) {
            row = table.getRowSorter().convertRowIndexToModel(row);
        }


        switch (column) {
            case 0:
                // scope
                BeanValidatorScope scope = (BeanValidatorScope) value;
                icon = SwingValidatorUtil.getIcon(scope);
                String label = _(scope.getLabel());
                toolTipText = _("validator.scope.tip", label);
                break;

            case 1:
                // field name
                text = getFieldName(table, (String) value, row);
                toolTipText = _("validator.field.tip", text);
                break;

            case 2:
                // message
                text = getMessage(table, (String) value, row);
                toolTipText = _("validator.message.tip", text);
                break;
        }

        rendererComponent.setText(text);
        rendererComponent.setToolTipText(toolTipText);
        rendererComponent.setIcon(icon);
        return rendererComponent;
    }

    public ImageIcon getIcon(BeanValidatorScope scope) {
        ImageIcon icon = SwingValidatorUtil.getIcon(scope);
        return icon;
    }

    public String getMessage(JTable table, String value, int row) {
        SwingValidatorMessageTableModel tableModel = (SwingValidatorMessageTableModel) table.getModel();
        SwingValidatorMessage model = tableModel.getRow(row);
        String text = SwingValidatorUtil.getMessage(model);
        return text;
    }

    public String getFieldName(JTable table, String value, int row) {
        SwingValidatorMessageTableModel tableModel = (SwingValidatorMessageTableModel) table.getModel();
        SwingValidatorMessage model = tableModel.getRow(row);
        String fieldName = SwingValidatorUtil.getFieldName(model, value);
        return fieldName;
    }
}